 '-----------------------------------------------------------------------
 '                      T E S T S C R E E N 3
 '-----------------------------------------------------------------------
 ' Performance test of 3 algorithms about string building.
 '-----------------------------------------------------------------------
 ' Input files: TestScreen.scn
 ' Process: Conversion from DOS format to WIN32 console format.
 '          The conversion is processed using 3 algorithms:
 '          - 1 - String concatenation and estraction with MID$
 '          - 2 - String allocation and bytes PEEKing/POKEing
 '          - 3 - String allocation and bytes copyng with ASM
 ' Output: Display screen TestScreen.scn on the WIN32 console
 '         and the number of cycles used by each algorithm.
 '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 ' Date: 23-apr-2010
 ' Language: PB/CC 5.04
 ' Author: frktons @ MASM32 Forum
 '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


 #INCLUDE "win32api.inc"


 DECLARE SUB ReadForm(BYVAL location AS LONG)

 DECLARE SUB DisplayScreen

 DECLARE SUB TestMID(BYVAL Qt AS LONG)
 DECLARE SUB TestPOKE(BYVAL Qt AS LONG)
 DECLARE SUB TestASM(BYVAL Qt AS LONG)


 FUNCTION PBMAIN()

   GLOBAL MainStr AS STRING, sBuf AS STRING
   GLOBAL QtConv AS LONG, location AS LONG

   CONSOLE SCREEN 25,80

   LOCAL hWin AS DWORD

again:

   QtConv = 0
   GRAPHIC WINDOW "Conversion Test for DOS to WIN32 Screens", 200, 200, 640, 80 TO hWin

   GRAPHIC ATTACH hWin, 0

   GRAPHIC INPUT "How many conversions to test?(1-1000) ", QtConv
   IF QtConv = 0 THEN
       GOTO exit_pgm
   END IF
   IF QtConv < 0 OR QtConv > 1000 THEN
       GOTO again
   END IF

   GRAPHIC WINDOW END

   CURSOR OFF
   COLOR 7,0
   CLS


   CALL ReadForm(1)
   CALL TestPOKE(1)
   CALL DisplayScreen
   LOCATE 2,71
   COLOR 7,0
   PRINT FORMAT$(QtConv,"###,");
   TIX cycles&&
       CALL TestPOKE(QtConv)
   TIX END cycles&&
   LOCATE 24,14
   PRINT FORMAT$(cycles&&,"###,");
   TIX cycles&&
       CALL TestMID(QtConv)
   TIX END cycles&&
   LOCATE 24,40
   PRINT FORMAT$(cycles&&,"###,");
   TIX cycles&&
       CALL TestASM(QtConv)
   TIX END cycles&&
   LOCATE 24,68
   PRINT FORMAT$(cycles&&,"###,");

   WAITKEY$
   GOTO again

 exit_pgm:

 END FUNCTION


'--------------------------------------------------------------------------------
' Display the converted Screen
'--------------------------------------------------------------------------------

 SUB DisplayScreen

    LOCAL lpReadRegion AS SMALL_RECT
    LOCAL SIZE AS DWORD
    SIZE = MAKDWD(80, 25)

    lpReadRegion.xRight = 79
    lpReadRegion.xBottom = 24
    WriteConsoleOutPut GetStdHandle(%STD_OUTPUT_HANDLE), _
    BYVAL STRPTR(sBuf),BYVAL SIZE, BYVAL 0&, lpReadRegion
 END SUB

'--------------------------------------------------------------------------------
' Read the DOS screen into the variabile MainStr
'--------------------------------------------------------------------------------

 SUB ReadForm(BYVAL location AS LONG)
    LOCAL ff AS LONG

    ff = FREEFILE
    OPEN "TestScreen.scn" FOR BINARY AS #ff
      SEEK ff, location
      GET$ #ff, 4000, MainStr
    CLOSE #ff

 END SUB

'--------------------------------------------------------------------------------
' Convert the DOS screen for the number of cycles chosen - PEEK/POKE version
'--------------------------------------------------------------------------------
  SUB TestPOKE(BYVAL cycles AS LONG)

    REGISTER x AS LONG
    REGISTER x1 AS LONG

    DIM y AS STRING PTR, y1 AS STRING PTR



    FOR x1 = 1 TO cycles
       sBuf  = SPACE$(8000)

       y = STRPTR(MainStr)
       y1 = STRPTR(sBuf)

       FOR x = 1 TO 8000 STEP 2
         POKE y1, PEEK(y)
         y = y + 1
         y1 = y1 + 2
       NEXT x
    NEXT x1


 END SUB

'--------------------------------------------------------------------------------
' Convert the DOS screen for the number of cycles chosen - MID$ version
'--------------------------------------------------------------------------------
  SUB TestMID(BYVAL cycles AS LONG)

    REGISTER x AS LONG
    REGISTER x1 AS LONG

    FOR x1 = 1 TO cycles
       sBuf = ""

       FOR x = 1 TO 4000
         sBuf = sBuf + MID$(MainStr, x, 1)+ " "
       NEXT x
    NEXT x1

  END SUB

'--------------------------------------------------------------------------------
' Convert the DOS screen for the number of cycles chosen - ASM version
'--------------------------------------------------------------------------------
 SUB TestASM(BYVAL cycles AS LONG)

    #REGISTER NONE

    DIM y AS STRING PTR, y1 AS STRING PTR
    DIM P_sBuf AS STRING PTR, P_MainStr AS STRING PTR
    DIM x AS LONG

 convert_main:


    ! push ebx
    ! push esi
    ! push edi
    ! push eax
    ! push edx

    sBuf = SPACE$(8000)
    P_sBuf = STRPTR(sBuf)
    P_MainStr = STRPTR(MainStr)

    ! mov edx,cycles

    main_cycle:

       y = P_MainStr
       y1 = P_sBuf


       ! mov ebx, 1000
       ! mov esi, y
       ! mov edi, y1
       Convert:
       ! mov eax, [esi]
       ! mov [edi], al
       ! mov [edi+2], ah
       ! bswap eax
       ! mov [edi+6], al
       ! mov [edi+4], ah
       ! add esi, 4
       ! add edi, 8
       ! dec ebx
       ! jnz Convert
       ! dec edx
       ! jnz main_cycle




    ! pop edx
    ! pop eax
    ! pop edi
    ! pop esi
    ! pop ebx

  END SUB
